@@ -62,7 +62,6 @@
 }
 
 GainDataPtr theGainData;
-static const uint32_t verbosity = MP4_DETAILS_ERROR|MP4_DETAILS_WARNING;
 
 //replay_gain tags
 static char *RGTags[num_rg_tags] = 
@@ -82,18 +81,15 @@ typedef struct
 } PreserveTimestamp, *PreserveTimestampPtr;
 
 //used to suppress bogus error messages from mp4lib
-static void mp4v2_lib_message_func(int loglevel, const char* lib, const char* fmt, ...)
+static void mp4v2_lib_message_func(MP4LogLevel loglevel, const char* fmt, va_list ap)
 {
 	char buf[512];
-	va_list ap;
-	va_start(ap, fmt);
 	vsnprintf(buf, sizeof(buf), fmt, ap);
 	//ignore this mp4v2 error message:
 	if (!strstr(buf, "no such property - moov.iods.audioProfileLevelId"))
 	{
 		fprintf(stderr, "mp4v2 error: %s\n", buf);
 	}
-	va_end(ap);
 }
 
 static char* temp_file_name(const char* input_file_name)
@@ -137,7 +133,7 @@ static char* temp_file_name(const char* input_file_name)
 
 static MP4FileHandle mp4_open_ro(GainDataPtr gd, char *mp4_file_name)
 {
-	MP4FileHandle fh = MP4Read(mp4_file_name, verbosity);
+	MP4FileHandle fh = MP4Read(mp4_file_name);
 	if (fh) 
 	{
 		gd->mp4File = fh;
@@ -148,7 +144,7 @@ static MP4FileHandle mp4_open_ro(GainDataPtr gd, char *mp4_file_name)
 		gd->itmfList = MP4ItmfGetItems(gd->mp4File);
 		if (!gd->itmfList)
 		{
-			MP4Close(fh);
+			MP4Close(fh, MP4_CLOSE_DO_NOT_COMPUTE_BITRATE);
 			return NULL;
 		}
 	}
@@ -158,7 +154,7 @@ static MP4FileHandle mp4_open_ro(GainDataPtr gd, char *mp4_file_name)
 static void mp4_open_rw(GainDataPtr gd, char *mp4_file_name)
 {
 	static const char *msg = "Unable to open file %s for writing.\n";
-	gd->mp4File = MP4Modify(gd->temp_name, verbosity);
+	gd->mp4File = MP4Modify(gd->temp_name, 0);
 	if (!gd->mp4File)
 	{
 		fprintf(stderr, msg, gd->temp_name);
@@ -176,7 +172,7 @@ static void mp4_close(GainDataPtr gd)
 {
 	if (gd->mp4File)
 	{
-		MP4Close(gd->mp4File);
+		MP4Close(gd->mp4File, MP4_CLOSE_DO_NOT_COMPUTE_BITRATE);
 		MP4ItmfItemListFree((MP4ItmfItemList *)gd->itmfList);
 		gd->mp4File = NULL;
 		gd->itmfList = NULL;
@@ -445,7 +441,7 @@ static int make_gain_adjustments(GainDataPtr gd, int left, int right)
 	return 0;
 }
 
-int aac_open(char *mp4_file_name, int use_temp, int preserve_timestamp, AACGainHandle *gh)
+int aac_open(char *mp4_file_name, int use_temp, int preserve_timestamp, int track_index, AACGainHandle *gh)
 {
     FILE* mp4_file;
     GainDataPtr gd;
@@ -506,20 +502,33 @@ int aac_open(char *mp4_file_name, int use_temp, int preserve_timestamp, AACGainH
     unsigned char *buffer = NULL;
     unsigned int buffer_size = 0;
 
-	MP4SetLibFunc(mp4v2_lib_message_func);
+	MP4SetLogCallback(&mp4v2_lib_message_func);
 
 	MP4FileHandle fh = mp4_open_ro(gd, mp4_file_name);
 	if (fh)
 	{
-        if (MP4GetNumberOfTracks(fh, MP4_AUDIO_TRACK_TYPE) != 1)
+        int num_audio_tracks = MP4GetNumberOfTracks(fh, MP4_AUDIO_TRACK_TYPE);
+        if (num_audio_tracks == 0)
         {
-            fprintf(stderr, "Error: File must contain a single audio track.\n");
+            fprintf(stderr, "Error: File must contain at least one audio track.\n");
 			gd->abort = 1;
             return 1;
         }
 
-		/* Find the first audio track, store in GainData struct. */
-		gd->track = MP4FindTrackId(fh, 0, MP4_AUDIO_TRACK_TYPE);
+        /* Print a warning if there are multiple audio tracks */
+        if (num_audio_tracks > 1)
+        {
+            fprintf(stderr, "File contains %d audio tracks, using track index %d. (Use -i option to override)\n", num_audio_tracks, track_index);
+        }
+
+		/* Find the audio track given by track_index.  Store in GainData struct. */
+		gd->track = MP4FindTrackId(fh, track_index, MP4_AUDIO_TRACK_TYPE);
+        if (gd->track == MP4_INVALID_TRACK_ID)
+        {
+            fprintf(stderr, "Error: Audio track index %d is not found in file.\n", track_index);
+            gd->abort = 1;
+            return 1;
+        }
 
 		//check for Apple Lossless
 		if (MP4HaveTrackAtom(fh, gd->track, "mdia.minf.stbl.stsd.alac"))
@@ -808,15 +817,21 @@ int aac_close(AACGainHandle gh)
         {
             //use MP4Optimize to undo the wasted space created by MP4File::Modify
             //send optimize output to a temp file "just in case"
-            MP4Optimize(gd->temp_name, temp_name);
-
-            //rename the temp file back to original name
-            int rc = remove(gd->mp4file_name);
-            if (rc == 0)
-                rc = rename(temp_name, gd->mp4file_name);
-            if (rc)
-                fprintf(stderr, "Error: attempt to create file %s failed. Your output file is named %s",
-                    gd->mp4file_name, temp_name);
+            if(MP4Optimize(gd->temp_name, temp_name)){
+                //rename the temp file back to original name
+                int rc = remove(gd->mp4file_name);
+                if (rc == 0)
+                    rc = rename(temp_name, gd->mp4file_name);
+                if (rc)
+                    fprintf(stderr, "Error: attempt to create file %s failed. Your output file is named %s",
+                        gd->mp4file_name, temp_name);
+            }
+            else{
+                //Optimize failed, keep the originals and remove temp files.
+                //The unoptimized temp file does not play in the app I tried it with, so you will not want it anyway.
+                fprintf(stderr, "Error: attempt to optimize file %s failed. The file was not modified.", gd->mp4file_name);
+                remove(temp_name); //Probably does not exist, but no harm in trying to remove it.
+            }
             free((void*)temp_name);
         }
         remove(gd->temp_name);
